/**
 * Copyright (C) 2010 Cloudfarming <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.sensor;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import javax.swing.filechooser.FileFilter;
import nl.cloudfarming.client.logging.AppLogFactory;
import nl.cloudfarming.client.logging.AppLogger;
import nl.cloudfarming.client.util.FilePickerUtil;
import org.apache.commons.io.FileUtils;
import org.netbeans.api.progress.ProgressHandle;
import org.netbeans.api.progress.ProgressHandleFactory;
import org.openide.awt.ActionID;
import org.openide.awt.ActionReference;
import org.openide.awt.ActionRegistration;
import org.openide.filesystems.FileChooserBuilder;
import org.openide.util.Exceptions;
import org.openide.util.Lookup;
import org.openide.util.LookupEvent;
import org.openide.util.LookupListener;
import org.openide.util.NbBundle;

/**
 *
 * @author Timon Veenstra
 */@ActionID(id = "nl.cloudfarming.client.sensor.SensorFilePicker", category = "Tools")
@ActionRegistration(iconInMenu = true, displayName = "#CTL_SensorFilePicker")
@ActionReference(path = "Menu/Tools", position = 50)

public class SensorFilePicker implements ActionListener {

    private final Lookup.Result<SensorProcessorProvider> sensorProcessorProviderResult = Lookup.getDefault().lookupResult(SensorProcessorProvider.class);
    private final ProxyFileFilter filterAllSensors = new ProxyFileFilter();
    private final String title = NbBundle.getMessage(this.getClass(), "sensor.filepicker.title");
    private final String approve = NbBundle.getMessage(this.getClass(), "sensor.filepicker.approve");
    private final String processProcessTitle = NbBundle.getMessage(this.getClass(), "sensor.process.process.title");
    private final String allSupportedFiletypeText = NbBundle.getMessage(this.getClass(), "sensor.filepicker.all_supported_filetypes.text");
         
    private final List<File> processQueue = Collections.synchronizedList(new ArrayList<File>());
    private final List<File> copyQueue = Collections.synchronizedList(new ArrayList<File>());
    private static final AppLogger LOG = AppLogFactory.getLogger(SensorFilePicker.class);

    public SensorFilePicker() {
        sensorProcessorProviderResult.addLookupListener(new LookupListener() {

            @Override
            public void resultChanged(LookupEvent ev) {
                synchronized (filterAllSensors) {
                    filterAllSensors.reset();
                    for (SensorProcessorProvider processorProvider : ((Lookup.Result<SensorProcessorProvider>) ev.getSource()).allInstances()) {
                        filterAllSensors.addFilter(FilePickerUtil.createSwingFileFilter(Arrays.asList(processorProvider.getSupportedExtentions()), processorProvider.getFormatDescription()) );
                    }
                }
            }
        });
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        File workingDirectory = new File(SensorPreference.LAST_DIR.getValue());

        FileChooserBuilder fcb = new FileChooserBuilder("Sensor-directory");
        fcb.setTitle(title);
        fcb.setDefaultWorkingDirectory(workingDirectory);
        fcb.setApproveText(approve);

        final ArrayList<String> supportedExtensions = new ArrayList<String>();
        for (SensorProcessorProvider processorProvider : Lookup.getDefault().lookupAll(SensorProcessorProvider.class)) {
            fcb.addFileFilter(FilePickerUtil.createSwingFileFilter(Arrays.asList(processorProvider.getSupportedExtentions()), processorProvider.getFormatDescription()));
            supportedExtensions.addAll(Arrays.asList(processorProvider.getSupportedExtentions()));
        }
        fcb.addFileFilter(FilePickerUtil.createSwingFileFilter(supportedExtensions, FilePickerUtil.getFileFilterDescription(allSupportedFiletypeText, supportedExtensions)));
        
        File selectedFile = fcb.showOpenDialog();

        if(selectedFile != null){
            if (selectedFile.isDirectory()) {
                synchronized (filterAllSensors) {
                    copyQueue.addAll(Arrays.asList(selectedFile.listFiles(filterAllSensors)));
                }
            } else if (selectedFile.isFile()) {
                copyQueue.add(selectedFile);
            }

        //        AggregateProgressHandle handle = AggregateProgressFactory.createHandle(processProcessTitle, new ProgressContributor[]{}, null, null);
        //        handle.start();
            //
            // start copy process
            //
            synchronized (processQueue) {
                File archiveDir = new File(SensorPreference.ARCHIVE.getValue());
                if (!archiveDir.exists()) {
                    archiveDir.mkdirs();
                }
                for (File file : copyQueue) {
                    try {
                        StringBuilder sb = new StringBuilder(archiveDir.getAbsolutePath());
                        sb.append("/");
                        sb.append(file.getName());
                        File archivedSensorFile = new File(sb.toString());
                        FileUtils.copyFile(file, archivedSensorFile, true);
                        processQueue.add(archivedSensorFile);

                    } catch (IOException ex) {
                        Exceptions.printStackTrace(ex);
                    }
                }
                copyQueue.clear();
            }
            //
            // start processing of files
            //
            Thread processThread = new Thread(new ProcessJob());
            processThread.start();
        }

    }

    private class ProxyFileFilter implements java.io.FileFilter {

        private final List<FileFilter> filters = new ArrayList<FileFilter>();

        @Override
        public boolean accept(File pathname) {
            for (FileFilter filter : filters) {
                if (filter.accept(pathname)) {
                    return true;
                }
            }
            return false;
        }

        public void reset() {
            filters.clear();
        }

        public void addFilter(FileFilter filter) {
            filters.add(filter);
        }
    }
    
    class ProcessJob implements Runnable {
        @Override
        public void run() {
            synchronized (processQueue) {
                // system handle will give this handle less priority diplaying file handles first
                ProgressHandle progressHandle = ProgressHandleFactory.createSystemHandle(processProcessTitle);
                int progress = 0;
                progressHandle.start(processQueue.size());
                for (File file : processQueue) {

                    findprocessor:
                    for (SensorProcessorProvider processorProvider : sensorProcessorProviderResult.allInstances()) {
                        if (processorProvider.isSupported(file) && processorProvider.isValid(file)) {
                            SensorProcessor processor = processorProvider.getProcessor(file);
                            if(processor != null){
                                processor.process();
                            }
                            else{
                                StringBuilder sb = new StringBuilder();
                                sb.append("ProcessorProvider [");
                                sb.append(processorProvider.getClass().toString());
                                sb.append("] did not provide a processor for file [");
                                sb.append(file.getName());
                                sb.append("] although this file IS valid and supported according to the processorprovider.(Bug?)");
                                LOG.warning(sb.toString());                 
                            }
                            progressHandle.progress(++progress);
                            break findprocessor;
                        }
                    }
                }
                progressHandle.finish();
                processQueue.clear();
            }
        }
    }
}
